#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyAlternate
  '$hullsaround_segs_outer','$hullsaround_segs_inner','$hullsaround_segs_all',
  'xy_to_node_name', '@dir4_to_dx','@dir4_to_dy', 'point_str_gp';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

print "  % generated by tools/$FindBin::Script\n";


my %seen;
foreach my $seg (sort $hullsaround_segs_all->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==1 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x,$y) [my$blank dot];\n";
    }
    $x += $dir4_to_dx[$dir];
    $y += $dir4_to_dy[$dir];
  }
}

my @draw;
foreach my $seg (sort $hullsaround_segs_outer->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  my $angle = 90*$dir;
  push @draw, "  \\draw[my arrow,my thick] ($from)--($to);\n";
  print "  \\pic at ($x,$y) [transform shape,rotate=$angle] {my hull pic};\n";
}
foreach my $seg (sort $hullsaround_segs_inner->difference(MyAlternateConf->initial)->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir4_to_dx[$dir], $y+$dir4_to_dy[$dir]);
  push @draw,  "  \\draw[my arrow] ($from)--($to);\n";
}
print @draw;

my @dir4_to_gp = ('1','I','-1','-I');
printwrap_indent("% GP-DEFINE");
$Printwrap = 2;
printwrap("fBpred_hullsaround_segs_outer = {[\n");
$Printwrap += 2;
my @segs_list = $hullsaround_segs_outer->segs_list;
foreach my $i (0 .. $#segs_list) {
  my ($x,$y,$dir) = split /,/, $segs_list[$i];
  printwrap(" [$x+I*$y,$dir4_to_gp[$dir]]" . ($i==$#segs_list ? '' : ','));
}
printwrap_newline();
$Printwrap = 2;
printwrap("]};\n");

print "  % end generated\n";
exit 0;
