#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate '@dir4_to_dx','@dir4_to_dy',
  'xy_rotate_minus90','xy_rotate_plus90','chamfer','turn';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $segments = MyLineSegments->new;
my %vertices = ('0,0' => 1);
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add ($path->n_to_xy($n), $path->n_to_xy($n+1));
  my ($x,$y) = $path->n_to_xy($n+1);
  $segments->add ($path->n_to_xy($n), $x,$y);
  $vertices{"$x,$y"} = 1;
}
my ($x_end,$y_end) = $path->n_to_xy($n_lo);
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap +=2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap = 4;
foreach my $xy (sort keys %vertices) {
  unless ($xy eq '0,0') {
    printwrap("($xy),");
  }
}
printwrap("} ");
printwrap("{ \\p node[my dot] };\n");
print "  \\node (start) at (0,0) [my dot];\n";

print "  % end generated\n";
exit 0;
