#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Graph::Maker::TwinAlternateAreaTree;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'TAVertexToZ_xy','mul';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

$Printwrap = 2;
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  printwrap("\\node at ($x,$y) [my dot,name=start];\n");
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = mul($x,$y);  # +45deg
  $x /= 2;
  $y /= 2;
  $y--;
  printwrap("\\node at ($x,$y) [my dot,name=end];\n");
}

my $segments = MyLineSegments->new;
my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1,
                              );
foreach my $edge ($graph->edges) {
  $segments->add(map {TAVertexToZ_xy($_)} @$edge);
}

$Printwrap = 2;
printwrap("\\draw[my tree] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

# $Printwrap = 2;
# printwrap("\\path foreach \\p in {");
# $Printwrap = 4;
# foreach my $xy (sort $graph->vertices) {
#   my ($x,$y) = split /,/, $xy;
#   printwrap("($x,$y),");
# }
# printwrap("} ");
# printwrap("{ \\p node[my vertex] };\n");

print "  % end generated\n";
exit 0;
