#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.006;
use strict;
use warnings;
use FindBin;
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh';

use lib $FindBin::Bin;
use MyAlternate 'TAparent','TAtospine','TAVertexToZ_xy',
  'TAVertexToUpper','TAVertexToLower',
  'TAVertexToLeft','TAVertexToRight';
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $n_hi = 2**$level - 1;

my @indomset = (1, 2, 4, 11, 13, 14);
my %indomset; @indomset{@indomset} = (); # hash slice

$Printwrap = 2;
print "  \\begin{scope}[font=\\scriptsize]\n";
foreach my $n (0 .. $n_hi) {
  my ($x,$y) = TAVertexToZ_xy($n);
  my $dot = (exists $indomset{$n} ? 'my solid dot' : 'my open dot');
  printwrap(" \\node (p$n) at ($x,$y) [$dot];");
}
printwrap_newline();
print "  \\end{scope}\n";

my $num_edges = 0;
$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $from (0 .. $n_hi) {
  foreach my $to (TAVertexToLeft($from),
                  TAVertexToRight($from),
                  TAVertexToLower($from),
                  TAVertexToUpper($from)) {
    if (defined $to
        && $to <= $n_hi
        && $to > $from) {
      printwrap(" (p$from)--(p$to)");
      $num_edges++;
    }
  }
}
printwrap(";\n");

$num_edges == 2**$level - 1
  or die "oops, num_edges=$num_edges";

my $varname = $FindBin::Script;
$varname =~ s/\.pl//;
$varname =~ tr/-/_/;
print "% GP-DEFINE  \\\\ with +1 for 1-based vpar\n";
print "% GP-DEFINE  ${varname}_set1 = ",
      "Set([", join(',',map{$_+1}@indomset), "]);\n";

print "% GP-Test  TAindomnum($level) == ",scalar(@indomset),"\n";
print "  % end generated\n";
exit 0;
