#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min';
use Math::PlanePath::AlternatePaper 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','xy_rotate_plus90','xy_rotate_minus90';
use MyPrintwrap ':all';

#   8 |                                                      128
#     |                                                       |
#   7 |                                                42---43/127
#     |                                                |      |
#   6 |                                         40---41/45--44/124
#     |                                         |      |      |
#   5 |                                  34---35/39--38/46--47/123
#     |                                  |      |      |      |
#   4 |                           32---33/53--36/52--37/49--48/112
#     |                           |      |      |      |      |
#   3 |                    10---11/31--30/54--51/55--50/58--59/111
#     |                    |      |      |      |      |      |
#   2 |              8----9/13--12/28--29/25--24/56--57/61--60/108
#     |              |     |      |      |      |      |      |
#   1 |        2----3/7---6/14--15/27--26/18--19/23---22/62--63/107
#     |        |     |     |      |      |      |      |      |
# Y=0 |  0-----1     4-----5     16-----17     20-----21     64---..
#
#  6 =  0110
# 22 = 10110
# 26 = 11010
# 6*4 = 24 = 11000


my $level = 14;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\node (start) at (0,0) [my dot];\n";
print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";

sub n_is_y1 {
  my ($n) = @_;
  my $factor = 1;
  for (;;) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    if ($y==1 && $dx==-1) { return $factor; }
    if ($n & 3) { return 0; }
    $factor *= 2;
    $n /= 4;
  }
}

$Printwrap = 2;
for (my $n=1; $n < $n_hi; ) {
  my $factor = n_is_y1($n);
  if ($factor) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($factor < 8) {
      print "  \\pic at ($x,$y) [transform shape] {my triangle pic={$factor,0}};\n";
    } else {
      print "  \\pic at ($x,$y) [transform shape] {my triangle gap pic={$factor,0}};\n";
      my $tx = $x-$factor;
      print "  \\draw[my line] ($x,$y) -- ($tx,$y);\n";
    }
    $n += $factor*$factor;
  } else {
    $n++;
  }
}

print "  % end generated\n";
exit 0;
